#include "Enemy.h"

#include "Player.h"
#include "LD37.h"

extern LD37* game;
extern int ENTCOUNT;

Enemy::Enemy(ALLEGRO_BITMAP* g) : Entity(g)
{
	liftable = false;
	solid = true;
	srcx = 16;
	srcy = 16;
	health = 2;
	php = 2;
	layer = 50;
	melee = true;
}

void Enemy::Update()
{
	if (!dead && !thrown)
	{
		bool dodge = false;
		if (rand() % 100 > 50)
		{
			dodge = true;
		}
		int adx = abs(game->ply->x - x);
		int ady = abs(game->ply->y - y);
		int dx = game->ply->x - x;
		if (dx != 0) dx = dx / abs(dx);
		int dy = game->ply->y - y;
		if (dy != 0) dy = dy / abs(dy);
		if (adx > 1 || ady > 1) 
		{
			if (!CanWalk(x + dx, y + dy, dodge))
			{
				if (adx > ady)
				{
					dy -= 1;
				}
				else
				{
					dx -= 1;
				}
			}
			if (!CanWalk(x + dx, y + dy, dodge))
			{
				if (adx > ady)
				{
					dy += 2;
				}
				else
				{
					dx += 2;
				}
			}
			if (CanWalk(x + dx, y + dy, dodge))
			{
				if (game->ply->x != x + dx || game->ply->y != y + dy)
				{
					x += dx;
					y += dy;
				}
			}
		}
		else
		{
			TryMove(dx, dy);
		}
		for (int i = 0; i < ENTCOUNT; ++i)
		{
			if (game->ents[i] != nullptr && game->ents[i] != this)
			{
				if (game->ents[i]->x == x && game->ents[i]->y == y && game->ents[i]->solid)
				{
					Bump(game->ents[i]);
					break;
				}
			}
		}
	}
}

void Enemy::Render()
{
	if ((x >= 2 && x <= 18 && y >= 2 && y <= 13)) 
	{
		if (flash > 0) flash -= 10;
		if (flash < 0) flash = 0;

		unsigned char v = 255 - flash;
		al_draw_tinted_bitmap_region(graphic, al_map_rgb(255, v, v), srcx, srcy, 16, 16, x * 16, y * 16, 0);
	}
}

void Enemy::Bump(Entity* perp)
{
	if (perp->thrown)
	{
		Fireball* f = dynamic_cast<Fireball*>(perp);
		if (f == NULL) 
		{
			Damage(perp->thdmg);
		}
	}
	Player* pl = dynamic_cast<Player*>(perp);
	if (pl != NULL)
	{
		Damage(1);
		if (game->keys[ALLEGRO_KEY_LCTRL])
		{
			int dx = x - pl->x;
			int dy = y - pl->y;
			TryMove(dx, dy);
		}
	}
}
void Enemy::Damage(int amt)
{
	
	health -= amt;
	flash = 255;
	if (health <= 0)
	{
		dead = true;
		solid = false;
		liftable = true;
		srcx = 32;
		srcy = 16;
		layer = 5;
		game->SortEntities();
		if (!game->p_death) { al_play_sample(game->s_death, 0.5f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0); game->p_death = true; }
	}
	else 
	{
		if (!game->p_hurt) 
		{
			al_play_sample(game->s_hurt, 0.5f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0); game->p_hurt = true;
		}
	}
	
}
Enemy::~Enemy()
{
}
