#include "Player.h"
#include "LD37.h"


extern LD37* game;
extern int ENTCOUNT;

Player::Player(ALLEGRO_BITMAP* g) : Entity(g)
{
	srcx = 0;
	srcy = 0;
	liftable = false;
	solid = true;
	health = 25;
	maxhealth = 25;
	hrt = false;
	layer = 1000;
	dynamites = 5;
	crystals = 0;
	

}

void Player::Update()
{
	if (pickup == nullptr) lifting = false;
	int px = x;
	int py = y;
	switch (game->key)
	{
	case ALLEGRO_KEY_W:
		TryMove(0, -1);
		dx = 0;
		dy = -1;
		break;
	case ALLEGRO_KEY_X:
		TryMove(0, 1);
		dx = 0;
		dy = 1;
		break;
	case ALLEGRO_KEY_D:
		TryMove(1, 0);
		dx = 1;
		dy = 0;
		break;
	case ALLEGRO_KEY_A:
		TryMove(-1, 0);
		dx = -1;
		dy = 0;
		break;
	case ALLEGRO_KEY_E:
		TryMove(1, -1);
		dx = 1;
		dy = -1;
		break;
	case ALLEGRO_KEY_Q:
		TryMove(-1, -1);
		dx = -1;
		dy = -1;
		break;
	case ALLEGRO_KEY_Z:
		TryMove(-1, 1);
		dx = -1;
		dy = 1;
		break;
	case ALLEGRO_KEY_C:
		TryMove(1, 1);
		dx = 1;
		dy = 1;
		break;
	case ALLEGRO_KEY_S:
		if (dynamites > 0) 
		{
			dynamites -= 1;
			Dynamite* dyn = new Dynamite(graphic);
			dyn->x = x;
			dyn->y = y;
			game->AddEnt(dyn);
			dyn = nullptr;
		}
		break;
	}
	if (game->keys[ALLEGRO_KEY_LSHIFT])
	{
		if (lifting) 
		{
			if (x != px || y != py) 
			{
				x = px;
				y = py;
				pickup->thrown = true;
				pickup->x = x + dx;
				pickup->y = y + dy;
				pickup->thx = dx;
				pickup->thy = dy;
				game->AddEnt(pickup);
				pickup = nullptr;
				lifting = false;
				if (!game->p_throw) { al_play_sample(game->s_throw, 0.5f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0); game->p_throw = true; }
			}
		}
		else
		{
			x = px;
			y = py;
			for (int i = 0; i < ENTCOUNT; ++i)
			{
				if (game->ents[i] != nullptr)
				{
					if (game->ents[i]->liftable) {
						if ((game->ents[i]->x == x + dx && game->ents[i]->y == y + dy)
							|| (game->ents[i]->x == x && game->ents[i]->y == y))
						{
							lifting = true;
							al_play_sample(game->s_pickup, 0.5f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0);
							Enemy* e = dynamic_cast<Enemy*>(game->ents[i]);
							if (e != NULL)
							{
								
								Barrel* b = new Barrel(graphic);
								b->srcx = 16;
								b->srcy = 16;
								Shrek* shr = dynamic_cast<Shrek*>(e);
								if (shr != NULL)
								{
									b->srcx = 64;
									b->srcy = 16;
								}
								shr = nullptr;
								pickup = b;
								game->RemoveEnt(game->ents[i], true);
							}
							else
							{
								pickup = game->ents[i];
								game->RemoveEnt(game->ents[i], false);
							}
							break;
						}
					}
				}
			}
			
		}
	}
	if (lifting)
	{
		srcx = 16;
		srcy = 0;
	}
	else
	{
		srcx = 0;
		srcy = 0;
	}
	for (int i = 0; i < ENTCOUNT; ++i)
	{
		if (game->ents[i] != nullptr)
		{
			if (game->ents[i]->x == x && game->ents[i]->y == y)
			{
				HPack* hp = dynamic_cast<HPack*>(game->ents[i]);
				if (hp != NULL)
				{
					health += 3;
					if (health > maxhealth) health = maxhealth;
					game->RemoveEnt(hp, true);
					if (!game->p_item) { al_play_sample(game->s_item, 0.5f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0); game->p_item = true; }
					continue;
				}
				else
				{
					DGet* d = dynamic_cast<DGet*>(game->ents[i]);
					if (d != NULL && dynamites < 9)
					{
						dynamites += 1;
						game->RemoveEnt(d, true);
						if (!game->p_item) { al_play_sample(game->s_item, 0.5f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0); game->p_item = true; }
						continue;
					}
					else
					{
						Crystal* c = dynamic_cast<Crystal*>(game->ents[i]);
						if (c != NULL)
						{
							crystals += 1;
							al_play_sample(game->s_crystal, 0.5f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0);
							game->screenFlash = 255;
							game->clearBoard = true;
							game->RemoveEnt(c, true);
							dynamites += 2;
							if (dynamites > 9) dynamites = 9;
							health += 10;
							if (health > maxhealth) health = maxhealth;
							continue;
						}
					}
				}
			}
		}
	}
}

void Player::Render()
{
	if (flash > 0) flash -= 10;
	if (flash < 0) flash = 0;

	unsigned char v = 255 - flash;
	al_draw_tinted_bitmap_region(graphic, al_map_rgb(255, v, v), srcx, srcy, 16, 16, x * 16, y * 16, 0);
	if (lifting)
	{
		if (pickup != nullptr) al_draw_tinted_scaled_rotated_bitmap_region(graphic, pickup->srcx, pickup->srcy, 16, 16, al_map_rgb(255,v,v),0,0, x*16, (y*16)+4, 1.0f, 1.0f, -3.14159f/2.0f, 0);
	}
}

void Player::Damage(int amt)
{
	if (!hrt) health -= amt;
	hrt = true;
	flash = 255;
	if (!game->p_hurt)
	{
		al_play_sample(game->s_hurt, 0.5f, 0.0f, 1.0f, ALLEGRO_PLAYMODE_ONCE, 0); game->p_hurt = true;
	}
}

void Player::Bump(Entity* perp)
{
	Enemy* enm = dynamic_cast<Enemy*>(perp);
	if (enm != NULL)
	{
		if (enm->melee) Damage(1);
	}
	else
	{
		Fireball* fb = dynamic_cast<Fireball*>(perp);
		if (fb != NULL)
		{
			Damage(1);
			//cout << "FIREBALL'D." << endl;
		}
	}
}

Player::~Player()
{
	if (pickup != nullptr)
	{
		delete pickup;
	}
}
